/* lzo1.h -- public interface of the LZO1 compression algorithm

   This file is part of the LZO real-time data compression library.

   Copyright (C) 1996 Markus Franz Xaver Johannes Oberhumer

   The LZO library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   The LZO library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the LZO library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation, Inc.,
   675 Mass Ave, Cambridge, MA 02139, USA.

   Markus F.X.J. Oberhumer
   markus.oberhumer@jk.uni-linz.ac.at
 */


#ifndef __LZO1_H
#define __LZO1_H

#include <lzoconf.h>

#ifdef __cplusplus
extern "C" {
#endif


/***********************************************************************
//
************************************************************************/

/* Memory required for the wrkmem parameter.
 * When the required size is 0, you can also pass a NULL pointer.
 */

#define LZO1_MEM_COMPRESS      ((lzo_uint) (8192L * sizeof(lzo_byte *)))
#define LZO1_MEM_DECOMPRESS    (0)


int lzo1_compress        ( const lzo_byte *src, lzo_uint  src_len,
                                 lzo_byte *dst, lzo_uint *dst_len,
                                 lzo_voidp wrkmem );

int lzo1_decompress      ( const lzo_byte *src, lzo_uint  src_len,
                                 lzo_byte *dst, lzo_uint *dst_len,
                                 lzo_voidp wrkmem /* NOT USED */ );

lzo_uint lzo1_info ( int *rbits, int *clevel );


/***********************************************************************
//
************************************************************************/

#define LZO1_99_MEM_COMPRESS   ((lzo_uint) (65536L * sizeof(lzo_byte *)))

int lzo1_99_compress     ( const lzo_byte *src, lzo_uint  src_len,
                                 lzo_byte *dst, lzo_uint *dst_len,
                                 lzo_voidp wrkmem );



#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* already included */

/*
vi:ts=4
*/
