/* lzo_test.c -- test driver for the LZO algorithm

   This file is part of the LZO real-time data compression library.

   Copyright (C) 1996 Markus Franz Xaver Johannes Oberhumer

   The LZO library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   The LZO library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the LZO library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation, Inc.,
   675 Mass Ave, Cambridge, MA 02139, USA.

   Markus F.X.J. Oberhumer
   markus.oberhumer@jk.uni-linz.ac.at
 */


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>
#include <assert.h>
#if defined(__DJGPP__) || defined(__BORLANDC__)
#  include <dir.h>
#endif
#if defined(__DJGPP__) || defined(__linux__)
#  include <unistd.h>
#endif

#include <lzoconf.h>


/*************************************************************************
//
**************************************************************************/

#if defined(ONLY_LZO1C_1)
#define HAVE_LZO1C_H
#else
#define HAVE_LZO1_H
#define HAVE_LZO1A_H
#define HAVE_LZO1B_H
#define HAVE_LZO1C_H
#define HAVE_LZO1D_H
#endif

#if defined(MFX) && !defined(ONLY_LZO1C_1)
#define HAVE_LZRW_H
#define HAVE_ZLIB_H
#define HAVE_LZV_H
#define HAVE_LZRW1O_H
#endif

#if 0
#define HAVE_LZO2A_H
#define HAVE_LZO2B_H
#define HAVE_LZO2C_H
#endif

#if 0
#define HAVE_LZO1X_H
#define HAVE_LZO4_H
#define HAVE_PRED_H
#endif



/*************************************************************************
// define all methods
**************************************************************************/

enum {
	M_LZO1B_1  =     1,
	M_LZO1B_2, M_LZO1B_3, M_LZO1B_4, M_LZO1B_5,
	M_LZO1B_6, M_LZO1B_7, M_LZO1B_8, M_LZO1B_9,

	M_LZO1C_1  =    11,
	M_LZO1C_2, M_LZO1C_3, M_LZO1C_4, M_LZO1C_5,
	M_LZO1C_6, M_LZO1C_7, M_LZO1C_8, M_LZO1C_9,

	M_LZO1     =    21,
	M_LZO1A    =    31,

	M_LZO1B_99 =    90,
	M_LZO1C_99 =    91,
	M_LZO1_99  =    92,
	M_LZO1A_99 =    93,
	M_LZO1C_999=    99,

	M_LZO1D_999=    41,
	M_LZO1X    =    51,
	M_LZO4     =   401,

	M_LZV      =  1001,
	M_PRED     =  1002,

	M_LZRW1A   =  1010,
	M_LZRW1O   =  1011,
	M_LZRW2    =  1020,
	M_LZRW3    =  1030,
	M_LZRW3A_2 =  1032,
	M_LZRW3A_4 =  1034,
	M_LZRW3A_8 =  1038, M_LZRW3A = M_LZRW3A_8,

	M_ZLIB_8_1 =  1101,
	M_ZLIB_8_2, M_ZLIB_8_3, M_ZLIB_8_4, M_ZLIB_8_5,
	M_ZLIB_8_6, M_ZLIB_8_7, M_ZLIB_8_8, M_ZLIB_8_9,

	M_MEMSET   =   998,
	M_MEMCPY   =   999,

	M_UNUSED
};



#if defined(__LZO_MSDOS16)
#define M_LZO1_99		(-1)
#define M_LZO1A_99		(-1)
#define M_LZO1B_99		(-1)
#define M_LZO1C_99		(-1)
#define M_LZO1C_999		(-1)

#undef HAVE_LZO1_H
#undef HAVE_LZO1A_H
//#undef HAVE_LZO1B_H
#if !defined(ONLY_LZO1C_1)
#undef HAVE_LZO1C_H
#endif
#undef HAVE_LZO1D_H

#undef HAVE_LZV_H
#undef HAVE_LZRW_H
#undef HAVE_ZLIB_H
#endif



/* LZO algorithms */
#if defined(HAVE_LZO1_H)
#  include <lzo1.h>
#endif
#if defined(HAVE_LZO1A_H)
#  include <lzo1a.h>
#endif
#if defined(HAVE_LZO1B_H)
#  include <lzo1b.h>
#endif
#if defined(HAVE_LZO1C_H)
#  include <lzo1c.h>
#endif
#if defined(HAVE_LZO1D_H)
#  include <lzo1d.h>
#endif
#if defined(HAVE_LZO1X_H)
#  include <lzo1x.h>
#endif
#if defined(HAVE_LZO2A_H)
#  include <lzo2a.h>
#endif
#if defined(HAVE_LZO2B_H)
#  include <lzo2b.h>
#endif
#if defined(HAVE_LZO2C_H)
#  include <lzo2c.h>
#endif
#if defined(HAVE_LZO4_H)
#  include <lzo4.h>
#endif

/* other realtime compressors */
#if defined(HAVE_LZRW_H)
#  include <lzrw.h>
#endif
#if defined(HAVE_LZRW1O_H)
#  include "contrib/lzrw1o/lzrw1o.h"
#endif
#if defined(HAVE_LZV_H)
#  include "contrib/lzv/lzv.h"
#endif
#if defined(HAVE_PRED_H)
#  include <pred.h>
#endif

/* other compressors */
#if defined(HAVE_ZLIB_H)
#  include <zlib.h>
#endif



#if defined(HAVE_LZO1B_H)
int method = M_LZO1B_1;
#else
int method = M_LZO1C_1;
#endif

/* set these to 1 to measure the speed impact of a checksum */
int compute_adler32 = 0;
int compute_crc32 = 0;


/*************************************************************************
//
**************************************************************************/

#if (UINT_MAX >= 0xffffffffL)		/* 32 bit or more */

#define BLOCK_SIZE		(256*1024l)		/* like in the LZRW test suite */
#define MAX_BLOCK_SIZE	(4*256*1024l)
#define DATA_LEN		(4*256*1024l)
#define WORK_LEN		(2*256*1024l)

#else								/* 16 bit compiler, oh my god */

#if 0
#define BLOCK_SIZE		(128*1024l)		/* like in the LZRW test suite */
#define MAX_BLOCK_SIZE	(256*1024l)
#define DATA_LEN		(128*1024l)
#else
#define BLOCK_SIZE		32000u
#define MAX_BLOCK_SIZE	32000u
#define DATA_LEN		(60*1024l)
#endif
#define WORK_LEN		(128*1024l)

#endif


static lzo_uint block_size = BLOCK_SIZE;

static lzo_byte block1[MAX_BLOCK_SIZE * 9L / 8 + 1024];
static lzo_byte block2[MAX_BLOCK_SIZE * 9L / 8 + 1024];

static lzo_byte data[DATA_LEN];

static lzo_byte wrkmem[WORK_LEN];


/*************************************************************************
// compress a block
**************************************************************************/

int
do_compress(const lzo_byte *in, lzo_byte *out,
                  lzo_uint in_len, lzo_uint *out_len)
{
	int r = -1;
	unsigned long o_len = ULONG_MAX;

	if (method < 0)
		;
#if defined(HAVE_LZO1_H)
	else if (method == M_LZO1)
		r = lzo1_compress(in,in_len,out,out_len,wrkmem);
#if (UINT_MAX >= 0xffffffffL)
	else if (method == M_LZO1_99)
		r = lzo1_99_compress(in,in_len,out,out_len,wrkmem);
#endif
#endif
#if defined(HAVE_LZO1A_H)
	else if (method == M_LZO1A)
		r = lzo1a_compress(in,in_len,out,out_len,wrkmem);
#if (UINT_MAX >= 0xffffffffL)
	else if (method == M_LZO1A_99)
		r = lzo1a_99_compress(in,in_len,out,out_len,wrkmem);
#endif
#endif
#if defined(HAVE_LZO1B_H)
	else if (method >= M_LZO1B_1 && method <= M_LZO1B_9)
	{
		int clevel = method - M_LZO1B_1 + 1;
		r = lzo1b_compress(in,in_len,out,out_len,wrkmem,clevel);
	}
#if (UINT_MAX >= 0xffffffffL)
	else if (method == M_LZO1B_99)
		r = lzo1b_99_compress(in,in_len,out,out_len,wrkmem);
#endif
#endif
#if defined(HAVE_LZO1C_H)
#if defined(ONLY_LZO1C_1)
	else if (method == M_LZO1C_1)
		r = lzo1c_1_compress(in,in_len,out,out_len,wrkmem);
#else
	else if (method >= M_LZO1C_1 && method <= M_LZO1C_9)
	{
		int clevel = method - M_LZO1C_1 + 1;
		r = lzo1c_compress(in,in_len,out,out_len,wrkmem,clevel);
	}
#if (UINT_MAX >= 0xffffffffL)
	else if (method == M_LZO1C_99)
		r = lzo1c_99_compress(in,in_len,out,out_len,wrkmem);
	else if (method == M_LZO1C_999)
		r = lzo1c_999_compress(in,in_len,out,out_len,wrkmem);
#endif
#endif
#endif
#if defined(HAVE_LZO1D_H)
	else if (method == M_LZO1D_999)
		r = lzo1d_999_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO1X_H)
	else if (method == M_LZO1X)
		r = lzo1x_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2A_H)
	else if (method == M_LZO2A)
		r = lzo2a_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2B_H)
	else if (method == M_LZO2B)
		r = lzo2b_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2C_H)
	else if (method == M_LZO2C)
		r = lzo2c_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO4_H)
	else if (method == M_LZO4)
		r = lzo4_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZRW_H)
	else if (method == M_LZRW1A)
		r = lzrw1a_compress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW2)
		r = lzrw2_compress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3)
		r = lzrw3_compress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_2)
		r = lzrw3a2_compress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_4)
		r = lzrw3a4_compress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_8)
		r = lzrw3a8_compress(wrkmem,in,in_len,out,&o_len);
#endif
#if defined(HAVE_LZRW1O_H)
	else if (method == M_LZRW1O)
		r = lzrw1o_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZV_H)
	else if (method == M_LZV)
		r = lzv_compress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_PRED_H)
	else if (method == M_PRED)
	{
		ipred(wrkmem);
		r = wpred(in,out,in_len,out_len);
	}
#endif
#if defined(HAVE_ZLIB_H)
	else if (method >= M_ZLIB_8_1 && method <= M_ZLIB_8_9)
	{
		/* use the undocumented feature to suppress the zlib header */
		z_stream stream;
		int err;
		int level = method - M_ZLIB_8_1 + 1;

		stream.next_in = (lzo_byte *) in;		/* UNCONST */
		stream.avail_in = in_len;
		stream.next_out = out;
		stream.avail_out = *out_len;

		stream.zalloc = (alloc_func)0;
		stream.zfree = (free_func)0;
		*out_len = 0;
#if 0
		err = deflateInit(&stream, level);
#else
		err = deflateInit2(&stream, level, Z_DEFLATED, -MAX_WBITS,
		                   MAX_MEM_LEVEL > 8 ? 8 : MAX_MEM_LEVEL,
		                   Z_DEFAULT_STRATEGY);
#endif
		if (err == Z_OK)
		{
			err = deflate(&stream, Z_FINISH);
			if (err != Z_STREAM_END)
			{
				deflateEnd(&stream);
				err = err == Z_OK ? Z_BUF_ERROR : err;
			}
			else
			{
				*out_len = (lzo_uint) stream.total_out;
				err = deflateEnd(&stream);
			}
		}
		r = err == Z_OK ? 0 : -1;
	}
#endif
	else if (method == M_MEMCPY)
	{
		lzo_memcpy(out,in,in_len);
		*out_len = in_len;
		r = 0;
	}

	if (o_len != ULONG_MAX)
		*out_len = (lzo_uint) o_len;

	if (r == 0 && compute_adler32)
	{
		lzo_uint adler;
		adler = lzo_adler32(0, NULL, 0);
		adler = lzo_adler32(adler, in, in_len);
	}
#if defined(HAVE_ZLIB_H)
	if (r == 0 && compute_crc32)
	{
		uLong crc;
		crc = crc32(0L, Z_NULL, 0);
		crc = crc32(crc, in, in_len);
	}
#endif

	return r;
}


/*************************************************************************
// decompress a block
**************************************************************************/

int 
do_decompress(const lzo_byte *in, lzo_byte *out,
              lzo_uint in_len, lzo_uint *out_len)
{
	int r = -1;
	unsigned long o_len = ULONG_MAX;

	if (method < 0)
		;
#if defined(HAVE_LZO1_H)
	else if (method == M_LZO1 || method == M_LZO1_99)
		r = lzo1_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO1A_H)
	else if (method == M_LZO1A || method == M_LZO1A_99)
		r = lzo1a_decompress(in,in_len,out,out_len,NULL);
#endif
#if defined(HAVE_LZO1B_H)
	else if (method >= M_LZO1B_1 && method <= M_LZO1B_9)
		r = lzo1b_decompress(in,in_len,out,out_len,wrkmem);
	else if (method == M_LZO1B_99)
		r = lzo1b_decompress_x(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO1C_H)
#if defined(ONLY_LZO1C_1)
	else if (method == M_LZO1C_1)
		r = lzo1c_decompress(in,in_len,out,out_len,wrkmem);
#else
	else if (method >= M_LZO1C_1 && method <= M_LZO1C_9)
		r = lzo1c_decompress(in,in_len,out,out_len,wrkmem);
	else if (method == M_LZO1C_99 || method == M_LZO1C_999)
		r = lzo1c_decompress_x(in,in_len,out,out_len,wrkmem);
#endif
#endif
#if defined(HAVE_LZO1D_H)
	else if (method == M_LZO1D_999)
		r = lzo1d_decompress_x(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO1X_H)
	else if (method == M_LZO1X)
		r = lzo1x_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2A_H)
	else if (method == M_LZO2A)
		r = lzo2a_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2B_H)
	else if (method == M_LZO2B)
		r = lzo2b_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO2C_H)
	else if (method == M_LZO2C)
		r = lzo2c_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZO4_H)
	else if (method == M_LZO4)
		r = lzo4_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZRW_H)
	else if (method == M_LZRW1A)
		r = lzrw1a_decompress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW2)
		r = lzrw2_decompress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3)
		r = lzrw3_decompress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_2)
		r = lzrw3a2_decompress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_4)
		r = lzrw3a4_decompress(wrkmem,in,in_len,out,&o_len);
	else if (method == M_LZRW3A_8)
		r = lzrw3a8_decompress(wrkmem,in,in_len,out,&o_len);
#endif
#if defined(HAVE_LZRW1O_H)
	else if (method == M_LZRW1O)
		r = lzrw1o_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_LZV_H)
	else if (method == M_LZV)
		r = lzv_decompress(in,in_len,out,out_len,wrkmem);
#endif
#if defined(HAVE_PRED_H)
	else if (method == M_PRED)
	{
		ipred(wrkmem);
		*out_len = rpred(in,out,in_len,*out_len);
		r = 0;
	}
#endif
#if defined(HAVE_ZLIB_H)
	else if (method >= M_ZLIB_8_1 && method <= M_ZLIB_8_9)
	{
		/* use the undocumented feature to suppress the zlib header */
		z_stream stream;
		int err;

		stream.next_in = (lzo_byte *) in;		/* UNCONST */
		stream.avail_in = in_len;
		stream.next_out = out;
		stream.avail_out = *out_len;

		stream.zalloc = (alloc_func)0;
		stream.zfree = (free_func)0;
		*out_len = 0;
#if 0
		err = inflateInit(&stream);
#else
		err = inflateInit2(&stream, -MAX_WBITS);
#endif
		if (err == Z_OK)
		{
			err = inflate(&stream, Z_FINISH);
			if (err == Z_OK)		/* sometimes returns Z_OK ??? */
			{
				*out_len = (lzo_uint) stream.total_out;
				err = inflateEnd(&stream);
			}
			else if (err == Z_STREAM_END)
			{
				*out_len = (lzo_uint) stream.total_out;
				err = inflateEnd(&stream);
			}
			else
				inflateEnd(&stream);
		}
		r = err == Z_OK ? 0 : -1;
	}
#endif
	else if (method == M_MEMCPY)
	{
		lzo_memcpy(out,in,in_len);
		*out_len = in_len;
		r = 0;
	}

	if (o_len != ULONG_MAX)
		*out_len = (lzo_uint) o_len;

	if (r == 0 && compute_adler32)
	{
		lzo_uint adler;
		adler = lzo_adler32(0, NULL, 0);
		adler = lzo_adler32(adler, in, in_len);
	}
#if defined(HAVE_ZLIB_H)
	if (r == 0 && compute_crc32)
	{
		uLong crc;
		crc = crc32(0L, Z_NULL, 0);
		crc = crc32(crc, in, in_len);
	}
#endif

	return r;
}


/*************************************************************************
// method info
**************************************************************************/

#define FPRINTF		(f == NULL) ? ((void) 0) : (void) fprintf

int info(FILE *f)
{
	if (method < 0)
		return 0;
#if defined(HAVE_LZO1_H)
	else if (method == M_LZO1)
	{
		int rbits, clevel;
		lzo1_info(&rbits,&clevel);
		FPRINTF(f,"LZO1-%d/%d",rbits,clevel);
	}
	else if (method == M_LZO1_99)
		FPRINTF(f,"LZO1-5/99");
#endif
#if defined(HAVE_LZO1A_H)
	else if (method == M_LZO1A)
	{
		int rbits, clevel;
		lzo1a_info(&rbits,&clevel);
		FPRINTF(f,"LZO1A-%d/%d",rbits,clevel);
	}
	else if (method == M_LZO1A_99)
		FPRINTF(f,"LZO1A-5/99");
#endif
#if defined(HAVE_LZO1B_H)
	else if (method >= M_LZO1B_1 && method <= M_LZO1B_9)
	{
		int clevel = method - M_LZO1B_1 + 1;
		FPRINTF(f,"LZO1B-%d",clevel);
	}
	else if (method == M_LZO1B_99)
		FPRINTF(f,"LZO1B-99");
#endif
#if defined(HAVE_LZO1C_H)
#if defined(ONLY_LZO1C_1)
	else if (method == M_LZO1C_1)
		FPRINTF(f,"LZO1C-1");
#else
	else if (method >= M_LZO1C_1 && method <= M_LZO1C_9)
	{
		int clevel = method - M_LZO1C_1 + 1;
		FPRINTF(f,"LZO1C-%d",clevel);
	}
	else if (method == M_LZO1C_99)
		FPRINTF(f,"LZO1C-99");
	else if (method == M_LZO1C_999)
		FPRINTF(f,"LZO1C-999");
#endif
#endif
#if defined(HAVE_LZO1D_H)
	else if (method == M_LZO1D_999)
		FPRINTF(f,"LZO1D-999");
#endif
#if defined(HAVE_LZO1X_H)
	else if (method == M_LZO1X)
	{
		lzo1x_info(&rbits,&clevel);
		FPRINTF(f,"LZO1X-%d/%d",rbits,clevel);
	}
#endif
#if defined(HAVE_LZO2A_H)
	else if (method == M_LZO2A)
	{
		lzo2a_info(&clevel);
		FPRINTF(f,"LZO2A-%d",clevel);
	}
#endif
#if defined(HAVE_LZO2B_H)
	else if (method == M_LZO2B)
	{
		lzo2b_info(&clevel);
		FPRINTF(f,"LZO2B-%d",clevel);
	}
#endif
#if defined(HAVE_LZO2C_H)
	else if (method == M_LZO2C)
	{
		lzo2c_info(&clevel,&ddbits);
		FPRINTF(f,"LZO2C-%d(%d)",clevel,ddbits);
	}
#endif
#if defined(HAVE_LZO4_H)
	else if (method == M_LZO4)
	{
		lzo4_info(&rbits,&clevel);
		FPRINTF(f,"LZO4-%d/%d",rbits,clevel);
	}
#endif
#if defined(HAVE_LZRW_H)
	else if (method == M_LZRW1A)
		FPRINTF(f,"LZRW1-A");
	else if (method == M_LZRW2)
		FPRINTF(f,"LZRW2");
	else if (method == M_LZRW3)
		FPRINTF(f,"LZRW3");
	else if (method == M_LZRW3A_2)
		FPRINTF(f,"LZRW3-A(2)");
	else if (method == M_LZRW3A_4)
		FPRINTF(f,"LZRW3-A(4)");
	else if (method == M_LZRW3A_8)
		FPRINTF(f,"LZRW3-A(8)");
#endif
#if defined(HAVE_LZRW1O_H)
	else if (method == M_LZRW1O)
		FPRINTF(f,"LZRW1-O");
#endif
#if defined(HAVE_LZV_H)
	else if (method == M_LZV)
		FPRINTF(f,"LZV");
#endif
#if defined(HAVE_PRED_H)
	else if (method == M_PRED)
		FPRINTF(f,"PRED");
#endif
#if defined(HAVE_ZLIB_H)
	else if (method >= M_ZLIB_8_1 && method <= M_ZLIB_8_9)
		FPRINTF(f,"zlib-%d/%d", Z_DEFLATED, method - M_ZLIB_8_1 + 1);
#endif
	else if (method == M_MEMCPY)
		FPRINTF(f,"memcpy");
	else if (f == NULL)
		return 0;
	else
	{
		FPRINTF(f,"invalid method %d !\n",method);
		exit(1);
	}

	return 1;
}

#undef FPRINTF


/*************************************************************************
// compress and decompress a file
**************************************************************************/

lzo_uint _lzo_fread(FILE *f, lzo_voidp s, lzo_uint len);

int do_file(const char *name, int loops, int c_loops, int d_loops, lzo_uint *ad)
{
	FILE *f;
	int i, r = 0;
	lzo_uint l = 0, t_len = 0, bytes = 0;
	int blocks = 0;
	clock_t t_time = 0, c_time = 0, d_time = 0;
	clock_t t_start, c_start, d_start;
	float secs, kbs;
	float perc, bits, c_kbs, d_kbs;
	lzo_uint adler;

	if (loops < 1)    loops = 1;
	if (c_loops < 1)  c_loops = 1;
	if (d_loops < 1)  d_loops = 1;


	/* read the whole file */
	f = fopen(name,"rb");
	if (f == NULL)
	{
		fprintf(stderr,"%s: ",name);
		perror("fopen");
		return 0;
	}
	l = _lzo_fread(f,data,DATA_LEN);
	if (fclose(f) != 0)
	{
		fprintf(stderr,"%s: ",name);
		perror("fclose");
		return -1;
	}

	adler = lzo_adler32(0, NULL, 0);
	adler = lzo_adler32(adler, data, l);
	if (ad)
		*ad = adler;

	printf("File %s: %lu bytes   adler32 = 0x%08lx\n", 
		name, (long) l, (long) adler);

	printf("  compressing %lu bytes (%d/%d/%d loops, %lu block-size)\n",
		(long) l, loops, c_loops, d_loops, (long) block_size);
	printf("  "); info(stdout); printf("\n");
	fflush(stdout);

	/* compress the file */

	t_start = clock();
	for (i = 0; i < loops; i++)
	{
		lzo_uint ll, c_len, c_len_max, d_len;
		lzo_byte *d = data;

		ll = l;
		t_len = 0;
		blocks = 0;
		while (ll > 0)
		{
			lzo_byte *dd = d;
			lzo_byte *b1 = block1;
			lzo_byte *b2 = block2;
			const lzo_uint bl = ll > block_size ? block_size : ll;
			int j;

			blocks++;

#if defined(__BOUNDS_CHECKING_ON)
			/* malloc a block of the exact block size to detect
			 * any overun.
			 */
			dd = malloc(bl);
			if (dd == NULL)
				return -1;
			memcpy(dd,d,bl);
			b2 = dd;
#endif

			/* compress a block */
			c_len_max = 0;
			c_start = clock();
			for (j = r = 0; r == 0 && j < c_loops; j++)
			{
				c_len = sizeof(block1);
				r = do_compress(dd,b1,bl,&c_len);
				if (r == 0 && c_len > c_len_max)
					c_len_max = c_len;
			}
			c_time += clock() - c_start;
			c_len = c_len_max;
			if (r != 0)
			{
				printf("  compression failed in block %d (%d) (%lu %lu)\n",
					blocks, r, (long)c_len, (long)bl);
				return -1;
			}

			/* decompress the block and verify */
			lzo_memset(b2,0,bl);
			d_start = clock();
			for (j = r = 0; r == 0 && j < d_loops; j++)
			{
				d_len = bl;
				r = do_decompress(b1,b2,c_len,&d_len);
			}
			d_time += clock() - d_start;
			if (r != 0)
			{
				printf("  decompression failed in block %d (%d) "
					"(%lu %lu %lu)\n", blocks, r,
					(long)c_len, (long)d_len, (long)bl);
				return -1;
			}
			if (d_len != bl)
			{
				printf("  decompression size error in block %d (%lu %lu %lu)\n",
					blocks, (long)c_len, (long)d_len, (long)bl);
				return -1;
			}
			if (lzo_memcmp(d,b2,bl) != 0)
			{
				lzo_uint x = 0;
				while (x < bl && d[x] == b2[x])
					x++;
				printf("  decompression data error in block %d at offset "
					"%lu (%lu %lu)\n", blocks, (long)x, (long)c_len,
					(long)d_len);
				return -1;
			}

#if defined(__BOUNDS_CHECKING_ON)
			if (dd != d)
				free(dd);
#endif
			d += bl;
			ll -= bl;
			t_len += c_len_max;

#if 0
			/* LZRW: add 8 bytes overhead per block to get
			 * the same values as published by Ross Williams
			 */
			if (method >= M_LZRW1A && method <= M_LZRW3A_8)
				t_len += 8;
#endif
		}
	}
	t_time += clock() - t_start;


	/* print some statistics */

	perc = (l > 0) ? t_len * 100.0 / l : 0;
	bits = perc * 0.08;
	printf("  compressed into %ld bytes, %.2f%%   %.3f\n",
		(long) t_len, perc, bits);

	secs = t_time / (float)(CLOCKS_PER_SEC);
	bytes = (l * c_loops + l * d_loops) * loops;
	kbs = (secs > 0.001) ? bytes / secs / 1000 : 0;
	printf("%-15s %5d: ","overall",loops);
	printf("%10lu bytes, %8.2f secs, %8.2f kB/sec\n", (long)bytes, secs, kbs);

	secs = c_time / (float)(CLOCKS_PER_SEC);
	bytes = l * c_loops * loops;
	c_kbs = (secs > 0.001) ? bytes / secs / 1000 : 0;
	printf("%-15s %5d: ","compress",c_loops);
	printf("%10lu bytes, %8.2f secs, %8.2f kB/sec\n", (long)bytes, secs, c_kbs);

	secs = d_time / (float)(CLOCKS_PER_SEC);
	bytes = l * d_loops * loops;
	d_kbs = (secs > 0.001) ? bytes / secs / 1000 : 0;
	printf("%-15s %5d: ","decompress",d_loops);
	printf("%10lu bytes, %8.2f secs, %8.2f kB/sec\n", (long)bytes, secs, d_kbs);


	/* create a line for util/table.pl */

#if defined(__DJGPP__) || defined(__EMX__) || defined(__linux__) || \
    defined(__BORLANDC__) || defined(__WATCOMC__)
	{
#if defined(__DJGPP__) || defined(__BORLANDC__)
		char n[256], ext[80]; fnsplit(name,NULL,NULL,n,ext); strcat(n,ext);
#elif defined(__EMX__) || defined(__WATCOMC__)
		char n[256], ext[80]; _splitpath(name,NULL,NULL,n,ext); strcat(n,ext);
#elif defined(__linux__)
		const char *n  = basename(name);
#endif
		printf("\n"); info(stdout);
		printf(" | %-14s %7ld %4d %7ld %6.1f %5.2f %8.2f %8.2f |\n",
			n, (long) l, blocks, (long) t_len, perc, bits, c_kbs, d_kbs);
		printf("\n");
	}
#endif

	return 0;
}


/*************************************************************************
// Calgary Corpus test suite
**************************************************************************/

static struct 
{
	const char *name;
	int loops;
	lzo_uint adler;
}
corpus[] =
{
	{ "bib",       70,  0x4bd09e98L },  
	{ "book1",     10,  0xd4d3613eL },
	{ "book2",     15,  0x6fe14cc3L },
	{ "geo",       60,  0xf3cc5be0L },
	{ "news",      20,  0x2ed405b8L },
	{ "obj1",     250,  0x3887dd2cL },
	{ "obj2",      50,  0xf89407c4L },
	{ "paper1",   130,  0xfe65ce62L },
	{ "paper2",   100,  0x1238b7c2L },
	{ "pic",       40,  0xf61a5702L },
	{ "progc",    200,  0x4c00ba45L },
	{ "progl",    200,  0x4cba738eL },
	{ "progp",    200,  0x7495b92bL },
	{ "trans",    100,  0x52a2cec8L }
};


int do_corpus(const char *path, int loops)
{
	int i, n;
	char name[256];

	if (path == NULL)
		return -1;

	strcpy(name,path);
	n = strlen(name);
	if (n > 0 && name[n-1] != '/' && name[n-1] != '\\')
	{
		strcat(name,"/");
		n++;
	}

	for (i = 0; i < (int) (sizeof(corpus)/sizeof(*(corpus))); i++)
	{
		lzo_uint adler;
		int l = loops * corpus[i].loops;

		strcpy(name+n,corpus[i].name);
		if (do_file(name, 1, l, 4*l, &adler) != 0)
			return -1;
		if (adler != corpus[i].adler)
			return -1;
	}
	return 0;
}



/*************************************************************************
// 
**************************************************************************/

static void usage(const char *name, int show_methods)
{
	FILE *f;
	int i;

	f = stdout;

	fprintf(f,"Usage: %s [-m#] [-b#] [-n#] files...\n", name);
	fprintf(f,"\n");
	fprintf(f,"Options:\n");
	fprintf(f,"  -m#     compression method\n");
	fprintf(f,"  -b#     set input block size (default %ld)\n",
		(long) block_size);
	fprintf(f,"  -n#     number of compression runs\n");

	if (show_methods)
	{
		fprintf(f,"\n");
		fprintf(f,"The following compression methods are available:\n");
		fprintf(f,"\n");

		for (i = 0; i < 2000; i++)
		{
			method = i;
			if (info(NULL))
			{
				char n[16];
				sprintf(n,"-m%d",i);
				fprintf(f,"  %-6s  ",n);
				info(f);
				fprintf(f,"\n");
			}
		}
	}
	else
	{
		fprintf(f,"\n");
		fprintf(f,"Type '%s -m' to list all available methods.\n", name);
	}

	exit(1);
}


/*************************************************************************
//
**************************************************************************/

int main(int argc, char *argv[])
{
	int i = 1;
	int loops = 1;
	int c_loops = 1;
	int d_loops = 1;
	int r = 0;
	const char *corpus_path = NULL;

#if defined(__EMX__)
	_response(&argc,&argv);
	_wildcard(&argc,&argv);
#endif

	printf("\nLZO real-time data compression library (v%s, %s).\n",
			LZO_VERSION_STRING, LZO_VERSION_DATE);
	printf("Copyright (C) 1996 Markus Franz Xaver Johannes Oberhumer\n\n");

	if (_lzo_config_check() != 0)
	{
		printf("Configuration check failed !!!\n");
		return 1;
	}

	if (argc < 2)
		usage(argv[0],0);
	
	if (i < argc  &&  isdigit(argv[i][0]))
		method = atoi(&argv[i++][0]);

	for ( ; i < argc && argv[i][0] == '-'; i++)
	{
		char *p = argv[i];

		if (isdigit(p[1]))
			method = atoi(p+1);
		else if (strncmp(p,"-m",2) == 0)
		{
			if (!isdigit(p[2]))
				usage(argv[0],1);
			method = atoi(p+2);
		}
		else if (strcmp(p,"-a") == 0)
			compute_adler32 = 1;
		else if (strncmp(p,"-b",2) == 0)
			block_size = atoi(p+2);
		else if (strncmp(p,"-c",2) == 0)
			loops = 1, c_loops = atoi(p+2), d_loops = 1;
		else if (strncmp(p,"-d",2) == 0)
			loops = 1, c_loops = 1, d_loops = atoi(p+2);
		else if (strncmp(p,"-n",2) == 0)
			loops = 1, c_loops = atoi(p+2), d_loops = c_loops;
		else if (strncmp(p,"-s",2) == 0)
		{
			if (i + 1 < argc)
			{
				loops = atoi(p+2);
				corpus_path = argv[++i];
			}
		}
		else
			usage(argv[0],0);
	}

	if (block_size < 512)
		block_size = 512;
	if (block_size > MAX_BLOCK_SIZE)
		block_size = MAX_BLOCK_SIZE;

	if (c_loops < 1)  c_loops = 10;
	if (d_loops < 1)  d_loops = 10;


	info(stdout); printf(" algorithm\n\n");

	if (corpus_path != NULL)
		r = do_corpus(corpus_path,loops);
	else
	{
		for ( ; i < argc; i++)
			r |= do_file(argv[i],loops,c_loops,d_loops,NULL);
	}

	return r == 0 ? 0 : 1;
}

/*
vi:ts=4
*/

